require 'prawn'
require 'prawn/security'
require "prawn/layout"
require 'barby'
require 'barby/outputter/png_outputter'
require 'barby/outputter/prawn_outputter'
require 'aws/s3'
require 'open-uri'

# fba_label = MWS::FBA::ShippingLabel.new({
#     :shipment_id => "FBA6PPQV5",
#     :ship_from_address => "Anton Zolotov\n1255 25th Street NW APT 405\nWashington, DC 20037\nUnited States",
#     :ship_to_address => "Amazon.com\n1221 24th Street NW, APT 228\nWashington, DC 20037\nUnited States",
#     :access_key_id => 'AKIAIDRBUMU6MCLORZ6Q',
#     :secret_access_key => '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',
#     :template_url => "http://bsgnpo.s3.amazonaws.com/template.fba.label.pdf",
#     :bucket => 'bsgnpo'
# })
# fba_label.build_pdf

module MWS    
  module FBA
    class ShippingLabel
      
      attr_accessor :url, :shipment_id, :ship_from_address, :ship_to_address, :access_key_id, :secret_access_key, :template_url, :bucket
      
      def initialize(args={})
        args.each { |k, v| send("#{k}=", v) }

        AWS::S3::Base.establish_connection!(
          :access_key_id => @access_key_id,
          :secret_access_key => @secret_access_key
        )
        upload
      end
      
      ##
      # Build PDF document and return Prawn::Document, ready to be saved locally or uploaded to S3
      # 
      def build_pdf
        Prawn::Document.new(:template => open(@template_url)) do |pdf|
          
          common_options = {
            :width => 200,
            :height => 50,
            :align => :left,
            :valign => :top,
            :overflow => :shrink_to_fit,
          }
          # print from address
          pdf.font "Helvetica", :size => 8
          pdf.text_box(@ship_from_address, common_options.merge({:at => [375,498]}))

          # print to address
          pdf.font "Helvetica", :style => :bold, :size => 11
          pdf.text_box(@ship_to_address, common_options.merge({:at => [540,498], :height => 40}))

          # print shipment ID
          pdf.font "Helvetica", :style => :bold, :size => 11
          puts pdf.text_box(@shipment_id, common_options.merge({
            :width    => 170,
            :height   => 16,
            :align    => :center,
            :at       => [455,374],
          }))

          # Generate barcode, store on s3, insert into pdf from S3
          barcode_png = Barby::Code128A.new(@shipment_id).to_png(:height => 50, :margin => 0)
          barcode_filename = "barcode_#{@shipment_id}.png"
          AWS::S3::S3Object.store(barcode_filename, barcode_png, @bucket)
          barcode_url = AWS::S3::S3Object.url_for(barcode_filename, @bucket, :expires_in => 3600)
          pdf.image open(barcode_url), :at => [470,430]
        end

      end
      
      def upload
        filename = "FBA_label_#{@shipment_id}.pdf"
        AWS::S3::S3Object.store(filename, build_pdf.render, @bucket)
        @url = AWS::S3::S3Object.url_for(filename, @bucket, :expires_in => 3600)
      end
      
    end
  end
end

