require 'prawn'
require 'prawn/security'
require "prawn/layout"
require 'barby'
require 'barby/outputter/png_outputter'
require 'barby/outputter/prawn_outputter'
require 'aws/s3'
require 'open-uri'

##
# packing_slip = MWS::PackingSlip.new({
#   :shipment_id => "FBA6PPQV5",
#   :ship_from_address => "Anton Zolotov\n1255 25th Street NW APT 405\nWashington, DC 20037\nUnited States",
#   :ship_to_address => "Amazon.com\n1221 24th Street NW, APT 228\nWashington, DC 20037\nUnited States",
#   :access_key_id => 'AKIAIDRBUMU6MCLORZ6Q',
#   :secret_access_key => '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',
#   :template_url => "http://bsgnpo.s3.amazonaws.com/template.fba.packing.slip.pdf",
#   :bucket => 'bsgnpo'
# })
# puts packing_slip.url
# 
module MWS
  class PackingSlip
    attr_accessor :url, :shipment_id, :ship_from_address, :ship_to_address, :access_key_id, :secret_access_key, :template_url, :bucket

    def initialize(args={})
      args.each { |k, v| send("#{k}=", v) }
      
      AWS::S3::Base.establish_connection!(
        :access_key_id => @access_key_id,
        :secret_access_key => @secret_access_key
      )
      
      upload
    end
    
    ##
    # Build PDF document and return Prawn::Document, ready to be saved locally or uploaded to S3
    # 
    def build_pdf
      Prawn::Document.new(:template => open(@template_url)) do |pdf|
        common_options = {
          :width => 200,
          :height => 50,
          :align => :left,
          :valign => :top,
          :overflow => :shrink_to_fit,
        }
        # print from address
        puts pdf.text_box(@ship_from_address, common_options.merge({:at => [95,392]}))
        
        # print to address
        pdf.text_box(@ship_to_address, common_options.merge({:at => [308,392]}))
        
        # print shipment ID
        pdf.font "Helvetica", :style => :bold, :size => 16
        pdf.text_box(@shipment_id, common_options.merge({
          :width    => 170,
          :height   => 16,
          :align    => :center,
          :at       => [185,250],
        }))
        
        # Generate barcode, store on s3, insert into pdf from S3
        barcode_png = Barby::Code128A.new(@shipment_id).to_png(:height => 60, :width => 142, :margin => 0)
        barcode_filename = "barcode_#{@shipment_id}.png"
        AWS::S3::S3Object.store(barcode_filename, barcode_png, @bucket)
        barcode_url = AWS::S3::S3Object.url_for(barcode_filename, @bucket, :expires_in => 3600)
        pdf.image open(barcode_url), :at => [200,315]
      end

    end
    
    ##
    # Upload packing slip to S3
    # 
    def upload
      filename = "packing_slip_#{@shipment_id}.pdf"
      AWS::S3::S3Object.store(filename, build_pdf.render, @bucket)
      @url = AWS::S3::S3Object.url_for(filename, @bucket, :expires_in => 3600)
    end
      
  end

end