require 'openssl'
require 'base64'
require 'httparty'

module MWS
  
  class Request
    include HTTParty
    base_uri 'https://mws.amazonservices.com'
    debug_output $stdout

    DEFAULTS = {
      "Version"           => "2010-10-01",
      "SignatureVersion"  => "2",
      "SignatureMethod"   => "HmacSHA256",
    }
    
    attr_accessor :parameters
    
    def initialize(parameters, secret_access_key)
      @parameters = parameters.merge!(DEFAULTS)
      @secret_access_key = secret_access_key
    end
    
    def <<(hash)
      @parameters.merge!(hash)
    end
    
    def post
      self.class.post('/FulfillmentInboundShipment/2010-10-01?' +  sign(build_query_string))
    end
    
    private
    
      def timestamp
        Time.now.utc.strftime '%Y-%m-%dT%H:%M:%SZ'
      end

      def escape(value)
        value.gsub(/([^a-zA-Z0-9_.~-]+)/) do
          '%' + $1.unpack('H2' * $1.bytesize).join('%').upcase
        end
      end

      def normalize
        @parameters.inject({}) do |hash, (k, v)|
          v = v.is_a?(Array) ? v.join(',') : v.to_s
          k = k.to_s.split('_').map {|w| w[0, 1] = w[0, 1].upcase; w }.join
          hash[k] = v
          hash
        end
      end

      def build_query_string
        normalize.merge('Timestamp' => timestamp).sort.map { |k, v| "#{k}=" + escape(v) }.join('&')
      end

      def sign(url)
        digest = OpenSSL::Digest::Digest.new('sha256')
        url_to_sign = [ "POST", "mws.amazonservices.com", '/FulfillmentInboundShipment/2010-10-01', url].join("\n")
        hmac = OpenSSL::HMAC.digest(digest, @secret_access_key, url_to_sign)
        url + "&Signature=" + escape([hmac].pack('m').chomp)
      end
      
  end
end